@@ -11,9 +11,21 @@
 #define EXTERN
 #endif
 
+/*
+ *
+ * Note: Each function declared EXTERN must both have 
+ *       an implementation in api.cpp, AND
+ *       a wrapper in ../combined/libssu.c
+ *
+ */
+
 #define PARTIAL_MAGIC "SSU-PARTIAL-01"
 #define PARTIAL_MAGIC_V2 0x088ABA02
 
+/*
+ * Set random seed used by this library.
+ */
+EXTERN void ssu_set_random_seed(unsigned int new_seed);
 
 /* a result matrix
  *
@@ -128,7 +140,7 @@ typedef struct partial_dyn_mat {
  * data <double*> the actual matrix values
  * n_obs <int> the number of observations, corresponding to length of obs_ids
  * n_samples <int> the number of samples, corresponding to the length of sample_ids
- * nnz <int> the number of nonzero values, corresponding to the length of data and indices
+ * nnz <int> the number of nonzero values, corresponding to the length of data and indices (DEPRECATED)
  */
 typedef struct support_biom {
     char** obs_ids;
@@ -196,6 +208,9 @@ EXTERN ComputeStatus one_off(const char* biom_filename, const char* tree_filenam
  * alpha <double> GUniFrac alpha, only relevant if method == generalized.
  * bypass_tips <bool> disregard tips, reduces compute by about 50%
  * n_substeps <uint> the number of substeps to use.
+ * subsample_depth <uint> Depth of subsampling, if >0
+ * subsample_with_replacement <bool> Use subsampling with replacement? (only True supported)
+ * mmap_dir <const char*> If not NULL, area to use for temp memory storage
  * result <mat_full_fp64_t**> the resulting distance matrix in full form, this is initialized within the method so using **
  *
  * one_off_inmem returns the following error codes:
@@ -204,6 +219,13 @@ EXTERN ComputeStatus one_off(const char* biom_filename, const char* tree_filenam
  * unknown_method : the requested method is unknown.
  * table_empty    : the table does not have any entries
  */
+EXTERN ComputeStatus one_off_matrix_inmem_v2(const support_biom_t *table_data, const support_bptree_t *tree_data,
+                                             const char* unifrac_method, bool variance_adjust, double alpha,
+                                             bool bypass_tips, unsigned int n_substeps,
+                                             unsigned int subsample_depth, bool subsample_with_replacement, const char *mmap_dir,
+                                             mat_full_fp64_t** result);
+
+/* Older version, will be deprecated in the future */
 EXTERN ComputeStatus one_off_inmem(const support_biom_t *table_data, const support_bptree_t *tree_data,
                                    const char* unifrac_method, bool variance_adjust, double alpha,
                                    bool bypass_tips, unsigned int n_substeps, mat_full_fp64_t** result);
@@ -217,6 +239,9 @@ EXTERN ComputeStatus one_off_inmem(const support_biom_t *table_data, const suppo
  * alpha <double> GUniFrac alpha, only relevant if method == generalized.
  * bypass_tips <bool> disregard tips, reduces compute by about 50%
  * n_substeps <uint> the number of substeps to use.
+ * subsample_depth <uint> Depth of subsampling, if >0
+ * subsample_with_replacement <bool> Use subsampling with replacement? (only True supported)
+ * mmap_dir <const char*> If not NULL, area to use for temp memory storage
  * result <mat_full_fp32_t**> the resulting distance matrix in full form, this is initialized within the method so using **
  *
  * one_off_inmem returns the following error codes:
@@ -225,6 +250,13 @@ EXTERN ComputeStatus one_off_inmem(const support_biom_t *table_data, const suppo
  * unknown_method : the requested method is unknown.
  * table_empty    : the table does not have any entries
  */
+EXTERN ComputeStatus one_off_matrix_inmem_fp32_v2(const support_biom_t *table_data, const support_bptree_t *tree_data,
+                                                  const char* unifrac_method, bool variance_adjust, double alpha,
+                                                  bool bypass_tips, unsigned int n_substeps,
+                                                  unsigned int subsample_depth, bool subsample_with_replacement, const char *mmap_dir,
+                                                  mat_full_fp32_t** result);
+
+/* Older version, will be deprecated in the future */
 EXTERN ComputeStatus one_off_inmem_fp32(const support_biom_t *table_data, const support_bptree_t *tree_data,
                                         const char* unifrac_method, bool variance_adjust, double alpha,
                                         bool bypass_tips, unsigned int n_substeps, mat_full_fp32_t** result);
@@ -238,6 +270,8 @@ EXTERN ComputeStatus one_off_inmem_fp32(const support_biom_t *table_data, const
  * alpha <double> GUniFrac alpha, only relevant if method == generalized.
  * bypass_tips <bool> disregard tips, reduces compute by about 50%
  * n_substeps <uint> the number of substeps/blocks to use.
+ * subsample_depth <uint> Depth of subsampling, if >0
+ * subsample_with_replacement <bool> Use subsampling with replacement? (only True supported)
  * mmap_dir <const char*> If not NULL, area to use for temp memory storage
  * result <mat_full_fp64_t**> the resulting distance matrix in matrix form, this is initialized within the method so using **
  *
@@ -249,6 +283,13 @@ EXTERN ComputeStatus one_off_inmem_fp32(const support_biom_t *table_data, const
  * unknown_method : the requested method is unknown.
  * table_empty    : the table does not have any entries
  */
+EXTERN ComputeStatus one_off_matrix_v2(const char* biom_filename, const char* tree_filename,
+                                       const char* unifrac_method, bool variance_adjust, double alpha,
+                                       bool bypass_tips, unsigned int n_substeps,
+                                       unsigned int subsample_depth, bool subsample_with_replacement, const char *mmap_dir,
+                                       mat_full_fp64_t** result);
+
+/* Older version, will be deprecated in the future */
 EXTERN ComputeStatus one_off_matrix(const char* biom_filename, const char* tree_filename,
                                     const char* unifrac_method, bool variance_adjust, double alpha,
                                     bool bypass_tips, unsigned int n_substeps,
@@ -264,6 +305,8 @@ EXTERN ComputeStatus one_off_matrix(const char* biom_filename, const char* tree_
  * alpha <double> GUniFrac alpha, only relevant if method == generalized.
  * bypass_tips <bool> disregard tips, reduces compute by about 50%
  * n_substeps <uint> the number of substeps/blocks to use.
+ * subsample_depth <uint> Depth of subsampling, if >0
+ * subsample_with_replacement <bool> Use subsampling with replacement? (only True supported)
  * mmap_dir <const char*> If not NULL, area to use for temp memory storage
  * result <mat_full_fp32_t**> the resulting distance matrix in matrix form, this is initialized within the method so using **
  *
@@ -275,13 +318,19 @@ EXTERN ComputeStatus one_off_matrix(const char* biom_filename, const char* tree_
  * unknown_method : the requested method is unknown.
  * table_empty    : the table does not have any entries
  */
+EXTERN ComputeStatus one_off_matrix_fp32_v2(const char* biom_filename, const char* tree_filename,
+                                            const char* unifrac_method, bool variance_adjust, double alpha,
+                                            bool bypass_tips, unsigned int n_substeps,
+                                            unsigned int subsample_depth, bool subsample_with_replacement, const char *mmap_dir,
+                                            mat_full_fp32_t** result);
+
+/* Older version, will be deprecated in the future */
 EXTERN ComputeStatus one_off_matrix_fp32(const char* biom_filename, const char* tree_filename,
                                          const char* unifrac_method, bool variance_adjust, double alpha,
                                          bool bypass_tips, unsigned int n_substeps,
                                          const char *mmap_dir,
                                          mat_full_fp32_t** result);
 
-
 /* compute Faith PD
  * biom_filename <const char*> the filename to the biom table.
  * tree_filename <const char*> the filename to the correspodning tree.
@@ -308,10 +357,15 @@ EXTERN ComputeStatus faith_pd_one_off(const char* biom_filename, const char* tre
  * bypass_tips <bool> disregard tips, reduces compute by about 50%
  * n_substeps <uint> the number of substeps to use.
  * format <const char*> output format to use.
+ * subsample_depth <uint> Depth of subsampling, if >0
+ * subsample_with_replacement <bool> Use subsampling with replacement? (only True supported)
  * pcoa_dims <uint> if not 0, number of dimensions to use or PCoA
+ * permanova_perms <uint> If not 0, compute PERMANOVA using that many permutations
+ * grouping_filename <const char*> the TSV filename containing grouping information
+ * grouping_columns <const char *> the columns to use for grouping
  * mmap_dir <const char*> if not empty, temp dir to use for disk-based memory 
  *
- * unifrac_to_file returns the following error codes:
+ * unifrac_to_file_v2 returns the following error codes:
  *
  * okay           : no problems encountered
  * table_missing  : the filename for the table does not exist
@@ -320,11 +374,97 @@ EXTERN ComputeStatus faith_pd_one_off(const char* biom_filename, const char* tre
  * table_empty    : the table does not have any entries
  * output_error   : failed to properly write the output file
  */
+EXTERN ComputeStatus unifrac_to_file_v2(const char* biom_filename, const char* tree_filename, const char* out_filename,
+                                        const char* unifrac_method, bool variance_adjust, double alpha,
+                                        bool bypass_tips, unsigned int n_substeps, const char* format,
+                                        unsigned int subsample_depth, bool subsample_with_replacement, 
+                                        unsigned int pcoa_dims,
+                                        unsigned int permanova_perms, const char *grouping_filename, const char *grouping_columns,
+                                        const char *mmap_dir);
+
+// backwards compatible version
 EXTERN ComputeStatus unifrac_to_file(const char* biom_filename, const char* tree_filename, const char* out_filename,
                                      const char* unifrac_method, bool variance_adjust, double alpha,
                                      bool bypass_tips, unsigned int n_substeps, const char* format,
                                      unsigned int pcoa_dims, const char *mmap_dir);
 
+/* Compute UniFrac with subsampling several times and save to file
+ *
+ * biom_filename <const char*> the filename to the biom table.
+ * tree_filename <const char*> the filename to the corresponding tree.
+ * out_filename <const char*> the filename of the output file.
+ * unifrac_method <const char*> the requested unifrac method.
+ * variance_adjust <bool> whether to apply variance adjustment.
+ * alpha <double> GUniFrac alpha, only relevant if method == generalized.
+ * bypass_tips <bool> disregard tips, reduces compute by about 50%
+ * n_substeps <uint> the number of substeps to use.
+ * format <const char*> output format to use.
+ * n_subsamples <uint> Number of subsamples to compute.
+ * subsample_depth <uint> Depth of subsampling.
+ * subsample_with_replacement <bool> Use subsampling with replacement? (only True supported)
+ * pcoa_dims <uint> if not 0, number of dimensions to use or PCoA
+ * permanova_perms <uint> If not 0, compute PERMANOVA using that many permutations
+ * grouping_filename <const char*> the TSV filename containing grouping information
+ * grouping_columns <const char *> the columns to use for grouping
+ * mmap_dir <const char*> if not empty, temp dir to use for disk-based memory 
+ *
+ * unifrac_multi_to_file_v2 returns the following error codes:
+ *
+ * okay           : no problems encountered
+ * table_missing  : the filename for the table does not exist
+ * tree_missing   : the filename for the tree does not exist
+ * unknown_method : the requested method is unknown.
+ * table_empty    : the table does not have any entries
+ * output_error   : failed to properly write the output file
+ */
+EXTERN ComputeStatus unifrac_multi_to_file_v2(const char* biom_filename, const char* tree_filename, const char* out_filename,
+                                              const char* unifrac_method, bool variance_adjust, double alpha,
+                                              bool bypass_tips, unsigned int n_substeps, const char* format,
+                                              unsigned int n_subsamples, unsigned int subsample_depth, bool subsample_with_replacement, 
+                                              unsigned int pcoa_dims,
+                                              unsigned int permanova_perms, const char *grouping_filename, const char *grouping_columns,
+                                              const char *mmap_dir);
+
+/* Compute PERMANOVA - fp64 variant
+ *
+ * grouping_filename <const char*> the filename to the grouping TSV file
+ * n_columns <uint> the number of grouping columns to process
+ * columns <const char**> the grouping columns, array of size n_columns
+ * mmap_dir <const char*> If not NULL, area to use for temp memory storage
+ * result <mat_full_fp64_t*> the distance matrix in matrix form
+ * permanova_perms <uint> the number of permutations, should be >0
+ * fstats <double *> the resulting PERMANOVA fstats, array of size n_columns
+ * pvalues <double *> the resulting PERMANOVA pvalues, array of size n_columns
+ *
+ * compute_permanova_fp64 returns the following error codes:
+ *
+ * okay               : no problems encountered
+ * grouping_missing  : the filename for the grouping does not exist or is not valid
+ */
+EXTERN ComputeStatus compute_permanova_fp64(const char *grouping_filename, unsigned int n_columns, const char* *columns,
+                                            mat_full_fp64_t * result, unsigned int permanova_perms,
+                                            double *fstats, double *pvalues);
+
+/* Compute PERMANOVA - fp32 variant
+ *
+ * grouping_filename <const char*> the filename to the grouping TSV file
+ * n_columns <uint> the number of grouping columns to process
+ * columns <const char**> the grouping columns, array of size n_columns
+ * mmap_dir <const char*> If not NULL, area to use for temp memory storage
+ * result <mat_full_fp32_t*> the distance matrix in matrix form
+ * permanova_perms <uint> the number of permutations, should be >0
+ * fstats <float *> the resulting PERMANOVA fstats, array of size n_columns
+ * pvalues <float *> the resulting PERMANOVA pvalues, array of size n_columns
+ *
+ * compute_permanova_fp32 returns the following error codes:
+ *
+ * okay               : no problems encountered
+ * grouping_missing  : the filename for the grouping does not exist or is not valid
+ */
+EXTERN ComputeStatus compute_permanova_fp32(const char *grouping_filename, unsigned int n_columns, const char* * columns,
+                                            mat_full_fp32_t * result, unsigned int permanova_perms,
+                                            float *fstats, float *pvalues);
+
 /* Write a matrix object
  *
  * filename <const char*> the file to write into
@@ -336,29 +476,33 @@ EXTERN ComputeStatus unifrac_to_file(const char* biom_filename, const char* tree
  */
 EXTERN IOStatus write_mat(const char* filename, mat_t* result);
 
-/* Write a matrix object using hdf5 format
+/* Write a matrix object using hdf5 format, using fp64 precision
  *
  * filename <const char*> the file to write into
  * result <mat_t*> the results object
  * pcoa_dims <uint> PCoAdimensions to compute, if >0
+ * save_dist <bool> If false, do not same the distance matrix data
  *
  * The following error codes are returned:
  *
  * write_okay : no problems
  */
-EXTERN IOStatus write_mat_hdf5(const char* filename, mat_t* result, unsigned int pcoa_dims);
+// backwards compatible version, deprecated
+IOStatus write_mat_hdf5_fp64(const char* filename, mat_t* result, unsigned int pcoa_dims, int save_dist);
 
 /* Write a matrix object using hdf5 format, using fp32 precision
  *
  * filename <const char*> the file to write into
  * result <mat_t*> the results object
  * pcoa_dims <uint> PCoAdimensions to compute, if >0
+ * save_dist <bool> If false, do not same the distance matrix data
  *
  * The following error codes are returned:
  *
  * write_okay : no problems
  */
-EXTERN IOStatus write_mat_hdf5_fp32(const char* filename, mat_t* result, unsigned int pcoa_dims);
+// backwards compatible version, deprecated
+IOStatus write_mat_hdf5_fp32(const char* filename, mat_t* result, unsigned int pcoa_dims, int save_dist);
 
 /* Write a matrix object
  *
@@ -372,29 +516,65 @@ EXTERN IOStatus write_mat_hdf5_fp32(const char* filename, mat_t* result, unsigne
 EXTERN IOStatus write_mat_from_matrix(const char* filename, mat_full_fp64_t* result);
 
 
-/* Write a matrix object from buffer using hdf5 format
+/* Write a matrix object from buffer using hdf5 format, using fp64 precision
  *
  * filename <const char*> the file to write into
  * result <mat_full_t*> the results object
- * pcoa_dims <uint> PCoAdimensions to compute, if >0
+ * pcoa_dims <uint> PCoA dimensions to compute, if >0
+ * save_dist <bool> If false, do not same the distance matrix data
+ * stat_n_vals <uint> Number of statistics to save
+ * stat_method_arr <const char**> Array of stat. methods names
+ * stat_name_arr   <const char**> Array of stat. test names
+ * stat_val_arr   <float*>        Array of stat. values
+ * stat_pval_arr  <float*>        Array of stat. p-values
+ * stat_perm_count_arr <uint*>    Array of the number of permutations used
+ * stat_group_name_arr <const char**> Array group names used for the grouping
+ * stat_group_count_arr <uint*>       Array number of groups used for the statistics
  *
  * The following error codes are returned:
  *
  * write_okay : no problems
+ * write_error : something went wrong
  */
-EXTERN IOStatus write_mat_from_matrix_hdf5(const char* filename, mat_full_fp64_t* result, unsigned int pcoa_dims);
+EXTERN IOStatus write_mat_from_matrix_hdf5_fp64_v2(const char* output_filename, mat_full_fp64_t* result,
+                                                   unsigned int pcoa_dims, int save_dist,
+                                                   unsigned int stat_n_vals,
+                                                   const char*  *stat_method_arr,     const char*        *stat_name_arr,
+                                                   const double *stat_val_arr,        const double       *stat_pval_arr, const unsigned int *stat_perm_count_arr,
+                                                   const char*  *stat_group_name_arr, const unsigned int *stat_group_count_arr);
+
+// backwards compatible version, deprecated
+EXTERN IOStatus write_mat_from_matrix_hdf5_fp64(const char* filename, mat_full_fp64_t* result, unsigned int pcoa_dims, int save_dist);
 
 /* Write a matrix object from buffer using hdf5 format, using fp32 precision
  *
  * filename <const char*> the file to write into
  * result <mat_full_fp32_t*> the results object
  * pcoa_dims <uint> PCoAdimensions to compute, if >0
+ * save_dist <bool> If false, do not same the distance matrix data
+ * stat_n_vals <uint> Number of statistics to save
+ * stat_method_arr <const char**> Array of stat. methods names
+ * stat_name_arr   <const char**> Array of stat. test names
+ * stat_val_arr   <float*>        Array of stat. values
+ * stat_pval_arr  <float*>        Array of stat. p-values
+ * stat_perm_count_arr <uint*>    Array of the number of permutations used
+ * stat_group_name_arr <const char**> Array group names used for the grouping
+ * stat_group_count_arr <uint*>       Array number of groups used for the statistics
  *
  * The following error codes are returned:
  *
  * write_okay : no problems
+ * write_error : something went wrong
  */
-EXTERN IOStatus write_mat_from_matrix_hdf5_fp32(const char* filename, mat_full_fp32_t* result, unsigned int pcoa_dims);
+EXTERN IOStatus write_mat_from_matrix_hdf5_fp32_v2(const char* output_filename, mat_full_fp32_t* result,
+                                                   unsigned int pcoa_dims, int save_dist,
+                                                   unsigned int stat_n_vals,
+                                                   const char*  *stat_method_arr,     const char*        *stat_name_arr,
+                                                   const float  *stat_val_arr,        const float        *stat_pval_arr, const unsigned int *stat_perm_count_arr,
+                                                   const char*  *stat_group_name_arr, const unsigned int *stat_group_count_arr);
+
+// backwards compatible version, deprecated
+EXTERN IOStatus write_mat_from_matrix_hdf5_fp32(const char* filename, mat_full_fp32_t* result, unsigned int pcoa_dims, int save_dist);
 
 /* Write a series
  *
@@ -446,7 +626,7 @@ EXTERN IOStatus write_vec(const char* filename, r_vec* result);
  * unknown_method : the requested method is unknown.
  */
 
-EXTERN ComputeStatus partial(const char* biom_filename, const char* tree_filename,
+ComputeStatus partial(const char* biom_filename, const char* tree_filename,
                              const char* unifrac_method, bool variance_adjust, double alpha,
                              bool bypass_tips, unsigned int n_substeps, unsigned int stripe_start,
                              unsigned int stripe_stop, partial_mat_t** result);
@@ -492,7 +672,7 @@ EXTERN ComputeStatus partial(const char* biom_filename, const char* tree_filenam
  * ### FOOTER ###
  * <MAGIC>              : char, e.g., SSU-PARTIAL-01, same as starting magic
  */
-EXTERN IOStatus write_partial(const char* filename, const partial_mat_t* result);
+IOStatus write_partial(const char* filename, const partial_mat_t* result);
 
 /* Read a partial matrix object
  *
@@ -507,7 +687,7 @@ EXTERN IOStatus write_partial(const char* filename, const partial_mat_t* result)
  * bad_header         : header seems malformed
  * unexpected_end     : format end not found in expected location
  */
-EXTERN IOStatus read_partial(const char* filename, partial_mat_t** result);
+IOStatus read_partial(const char* filename, partial_mat_t** result);
 
 /* Read a partial matrix object header
  *
@@ -522,7 +702,7 @@ EXTERN IOStatus read_partial(const char* filename, partial_mat_t** result);
  * bad_header         : header seems malformed
  * unexpected_end     : format end not found in expected location
  */
-EXTERN IOStatus read_partial_header(const char* input_filename, partial_dyn_mat_t** result_out);
+IOStatus read_partial_header(const char* input_filename, partial_dyn_mat_t** result_out);
 
 /* Read a stripe of a partial matrix
  *
@@ -538,12 +718,12 @@ EXTERN IOStatus read_partial_header(const char* input_filename, partial_dyn_mat_
  * bad_header         : header seems malformed
  * unexpected_end     : format end not found in expected location
  */
-EXTERN IOStatus read_partial_one_stripe(partial_dyn_mat_t* result, uint32_t stripe_idx);
+IOStatus read_partial_one_stripe(partial_dyn_mat_t* result, uint32_t stripe_idx);
 
 /*
  * Description TBD
  */
-EXTERN MergeStatus validate_partial(const partial_dyn_mat_t* const * partial_mats, int n_partials);
+MergeStatus validate_partial(const partial_dyn_mat_t* const * partial_mats, int n_partials);
 
 /* Merge partial results
  *
@@ -558,7 +738,7 @@ EXTERN MergeStatus validate_partial(const partial_dyn_mat_t* const * partial_mat
  * sample_id_consistency : samples described by stripes are inconsistent
  * square_mismatch       : inconsistency on denotation of square matrix
  */
-EXTERN MergeStatus merge_partial(partial_mat_t** partial_mats, int n_partials, unsigned int dummy, mat_t** result);
+MergeStatus merge_partial(partial_mat_t** partial_mats, int n_partials, unsigned int dummy, mat_t** result);
 
 /* Merge partial results
  *
