@@ -55,9 +55,10 @@ public:
   /* default constructor
    *
    * @param parent biom object to subsample
+   * @param w_replacement Whether to permute or use multinomial sampling
    * @param n Number of items to subsample
    */
-   skbio_biom_subsampled(const biom_inmem &parent, const uint32_t n);
+   skbio_biom_subsampled(const biom_inmem &parent, const bool w_replacement, const uint32_t n);
 };
 
 
